// Select Address Book Text - javascript overlay
//
// Context menu stuff is borrowed from nsContextMenu.js, with massive changes

//////////////////////////////////////////////////////////////
// Beginning of context menu stuff
var gContextMenu = null;

// Not clear why this is a separate initializer function
function selectAddressBookContextMenu(aXulMenu) {
  this.target         = null;
  this.onLink         = false;
  this.onMailtoLink   = false;
  this.link           = false;
  this.linkURL        = "";
  this.linkURI        = null;
  this.linkProtocol   = null;
  this.isContentSelected = false;
  this.targetId       = "";
  this.isName         = false;
  this.isHomeAddress  = false;
  this.isWorkAddress  = false;

  this.initMenu(aXulMenu);
}


// Prototype class for selectAddressBookContextMenu class
selectAddressBookContextMenu.prototype = {
  /**
   * Init: set properties based on the clicked-on element and the state of
   * the world, then determine which context menu items to show based on
   * those properties.
   */
  initMenu : function SAB_initMenu(aPopup) {
    this.menu = aPopup;

    // Get contextual info.
    this.setTarget(document.popupNode);
    this.isContentSelected = this.isContentSelection();

    this.initItems();
  },
  initItems : function SAB_initItems() {
    this.initClipboardItems();
  },
  initClipboardItems : function SAB_initClipboardItems() {
    // Copy depends on whether there is selected text.
    // The following filters enable each context menu item one-by-one

    // Copy selected text
    this.showItem("context-selectAddressBook-copy", 
		  this.isContentSelected || this.onTextInput);
    // Copy email or link
    this.showItem("context-selectAddressBook-copyemail", this.onMailtoLink);
    this.showItem("context-selectAddressBook-copylink", this.onLink);

    // Copy name and/or address
    this.showItem("context-selectAddressBook-copyDisplayNameFL", this.isName);

    this.showItem("context-selectAddressBook-copyFullHomeAddress", 
		  this.isHomeAddress);
    this.showItem("context-selectAddressBook-copyNameHomeAddress", 
		  this.isHomeAddress);
    this.showItem("context-selectAddressBook-copyFullWorkAddress", 
		  this.isWorkAddress);
    this.showItem("context-selectAddressBook-copyNameWorkAddress", 
		  this.isWorkAddress);

    // Copy phone numbers
    this.showItem("context-selectAddressBook-copyPhWork", 
		  this.targetId == "cvPhWork");
    this.showItem("context-selectAddressBook-copyPhHome", 
		  this.targetId == "cvPhHome");
    this.showItem("context-selectAddressBook-copyPhFax", 
		  this.targetId == "cvPhFax");
    this.showItem("context-selectAddressBook-copyPhCellular", 
		  this.targetId == "cvPhCellular");
    this.showItem("context-selectAddressBook-copyPhPager", 
		  this.targetId == "cvPhPager");

    // Copy other fields
    this.showItem("context-selectAddressBook-copyCustom1", 
		  this.targetId == "cvCustom1");
    this.showItem("context-selectAddressBook-copyCustom2", 
		  this.targetId == "cvCustom2");
    this.showItem("context-selectAddressBook-copyCustom3", 
		  this.targetId == "cvCustom3");
    this.showItem("context-selectAddressBook-copyCustom4", 
		  this.targetId == "cvCustom4");

    this.showItem("context-selectAddressBook-copyNotes", 
		  this.targetId == "cvNotes");
    this.showItem("context-selectAddressBook-copyNickName", 
		  this.targetId == "cvNickname");
    // this.showItem("context-selectAddressBook-copy", 
    //      this. == "cv");
  },

  /**
   * Set the nsContextMenu properties based on the selected node and
   * its ancestors.
   */
  setTarget : function SAB_setTarget(aNode) {
    // Email links and URLs are normal XHTML namespace nodes, while
    // other text nodes are stored under XUL namespace.
    const xulNS =
      "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    if (aNode.namespaceURI == xulNS) {
	// XUL namespace
	// fall through...
    } else {
      this.onLink         = false;
      this.linkURL        = "";
      this.linkURI        = null;
      this.linkProtocol   = null;
    }
      
    this.target = aNode;

    // Bubble out, looking for items of interest that might be
    // parents of the click target, picking the innermost of each.
    var elem = this.target;
    while (elem) {
      // Record which DESCRIPTION tag was hit, but only record the 
      // deepest element. --> elem.targetId
      var id = elem.id;
      if (this.targetId == "" && id && elem.nodeName == "description") {
	  this.targetId = id;
      }

      // Check for link types
      if (elem.nodeType == Node.ELEMENT_NODE) {
        // Link?
        if (!this.onLink && 
	    (elem.nodeName == "html:a" || elem.nodeName == "a")) {
          // Target is a link or a descendant of a link.
          this.onLink = true;
          // Remember corresponding element.
          this.link = elem;
          this.linkURL = this.getLinkURL();
          this.linkURI = this.getLinkURI();
          this.linkProtocol = this.getLinkProtocol();
          this.onMailtoLink = (this.linkProtocol == "mailto");
        }

      }
      elem = elem.parentNode;
    }

    // After bubbling out, we test for special types of fields
    // For example, isHomeAddress is set if any of the address fields
    // are hit.
    if (this.targetId != "") {
	this.isName = (this.targetId == "cvDisplayName");
	this.isHomeAddress = ((this.targetId == "cvHomeAddress") ||
			   (this.targetId == "cvHomeAddress2") ||
			   (this.targetId == "cvHomeCityStZip") ||
			   (this.targetId == "cvHomeCountry"));
	this.isWorkAddress = ((this.targetId == "cvJobTitle") ||
			   (this.targetId == "cvDepartment") ||
			   (this.targetId == "cvCompany") ||
			   (this.targetId == "cvWorkAddress") ||
			   (this.targetId == "cvWorkAddress2") ||
			   (this.targetId == "cvWorkCityStZip") ||
			   (this.targetId == "cvWorkCountry"));
    }    

  },

  /**
   * Get a computed style property for an element.
   * @param  aElem
   *         A DOM node
   * @param  aProp
   *         The desired CSS property
   * @return the value of the property
   */
  getComputedStyle: function SAB_getComputedStyle(aElem, aProp) {
    return aElem.ownerDocument.defaultView.getComputedStyle(aElem, "")
                .getPropertyValue(aProp);
  },

  /**
   * Generate a URL string from a computed style property, for things like
   * |style="background-image:url(...)"|
   * @return a "url"-type computed style attribute value, with the "url(" and
   *         ")" stripped.
   */
  getComputedURL: function SAB_getComputedURL(aElem, aProp) {
    var url = aElem.ownerDocument.defaultView.getComputedStyle(aElem, "")
                   .getPropertyCSSValue(aProp);
    return (url.primitiveType == CSSPrimitiveValue.CSS_URI) ? url.getStringValue() : null;
  },


  /**
   * Extract email addresses from a mailto: link and put them on the
   * clipboard.
   */
  copyEmail : function SAB_copyEmail() {
    // Copy the comma-separated list of email addresses only.
    // There are other ways of embedding email addresses in a mailto:
    // link, but such complex parsing is beyond us.

    const kMailToLength = 7; // length of "mailto:"

    var url = this.linkURL;
    var qmark = url.indexOf("?");
    var addresses;

    if (qmark > kMailToLength) {
      addresses = url.substring(kMailToLength, qmark);
    } else {
      addresses = url.substr(kMailToLength);
    }

    // Let's try to unescape it using a character set.
    try {
      var characterSet = this.target.ownerDocument.characterSet;
      const textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"]
                                     .getService(Components.interfaces.nsITextToSubURI);
      addresses = textToSubURI.unEscapeURIForUI(characterSet, addresses);
    }
    catch(ex) {
      // Do nothing.
    }

    var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                              .getService(Components.interfaces.nsIClipboardHelper);
    clipboard.copyString(addresses);
  },

  ///////////////
  // Utilities //
  ///////////////

  /**
   * Set a DOM node's hidden property by passing in the node's id or the
   * element itself.
   * @param aItemOrId
   *        a DOM node or the id of a DOM node
   * @param aShow
   *        true to show, false to hide
   */
  showItem : function SAB_showItem(aItemOrId, aShow) {
    var item = aItemOrId.constructor == String ? document.getElementById(aItemOrId) : aItemOrId;
    item.hidden = !aShow;
  },

  /**
   * Set given attribute of specified context-menu item. If the
   * value is null, then it removes the attribute (which works
   * nicely for the disabled attribute).
   * @param  aId
   *         The id of an element
   * @param  aAttr
   *         The attribute name
   * @param  aVal
   *         The value to set the attribute to, or null to remove the attribute
   */
  setItemAttr : function SAB_setItemAttr(aId, aAttr, aVal) {
    var elem = document.getElementById(aId);
    if (elem) {
      if (aVal == null) {
        // null indicates attr should be removed.
        elem.removeAttribute(aAttr);
      } else {
        // Set attr=val.
        elem.setAttribute(aAttr, aVal);
      }
    }
  },

  /**
   * Get an absolute URL for clicked-on link, from the href property or by
   * resolving an XLink URL by hand.
   * @return the string absolute URL for the clicked-on link
   */
  getLinkURL : function SAB_getLinkURL() {
    if (this.link.href) {
      return this.link.href;
    }
    var href = this.link.getAttributeNS("http://www.w3.org/1999/xlink","href");
    if (!href || !href.match(/\S/)) {
       // Without this we try to save as the current doc,
       // for example, HTML case also throws if empty.
      throw "Empty href";
    }
    href = this.makeURLAbsolute(this.link.baseURI,href);
    return href;
  },

  /**
   * Generate a URI object from the linkURL spec
   * @return an nsIURI if possible, or null if not
   */
  getLinkURI: function SAB_getLinkURI() {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
    try {
      return ioService.newURI(this.linkURL, null, null);
    } catch (ex) {
      // e.g. empty URL string
    }
    return null;
  },

  /**
   * Get the scheme for the clicked-on linkURI, if present.
   * @return a scheme, possibly undefined, or null if there's no linkURI
   */
  getLinkProtocol: function SAB_getLinkProtocol() {
    if (this.linkURI)
      return this.linkURI.scheme; // can be |undefined|

    return null;
  },

  /**
   * Determines whether the focused window has something selected.
   * @return true if there is a selection, false if not
   */
  isContentSelection : function SAB_isContentSelection() {
    return !document.commandDispatcher.focusedWindow.getSelection().isCollapsed;
  },

  /**
   * Convert relative URL to absolute, using a provided <base>.
   * @param  aBase
   *         The URL string to use as the base
   * @param  aUrl
   *         The possibly-relative URL string
   * @return The string absolute URL
   */
  makeURLAbsolute : function SAB_makeURLAbsolute(aBase, aUrl) {
    // Construct nsIURL.
    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
    var baseURI  = ioService.newURI(aBase, null, null);

    return ioService.newURI(baseURI.resolve(aUrl), null, null).spec;
  }
};


// 
// Called when context menu is hidden
// 
function selectAddressBookContextHiding()
{
  gContextMenu = null;
}

//
// Called when context menu is about to be shown
//     (creates a new context menu)
//
function selectAddressBookContextShowing(aTarget)
{
  gContextMenu = new selectAddressBookContextMenu(aTarget);
}

//////////////////////////////////////////////////////////////
// Utility routine - to retrieve address label format string
// 
var sab_strings;           // Stringbundle data obtained? (1=yes)
var sab_home_address_fmt;  // Home address format
var sab_work_address_fmt;  // Work address format
function sab_address_fmt(type) 
{

    if (sab_strings == undefined) {
	var strbundle = null;
	var f = (function(str) { try { return strbundle.getString(str); } 
		                 catch (e) { return undefined; }});

	sab_strings = 1;
	try {
	    strbundle = document.getElementById("sab_address_strings");
	} catch(e) {};

	if (strbundle && f("home_address_format1")) {
	    sab_home_address_fmt = [
				f("home_address_format1"),
				f("home_address_format2"),
				f("home_address_format3"),
				f("home_address_format4"),
				f("home_address_format5"),
				f("home_address_format6"),
				f("home_address_format7"),
				f("home_address_format8"),
				f("home_address_format9")];
	    sab_work_address_fmt = [
				f("work_address_format1"),
				f("work_address_format2"),
				f("work_address_format3"),
				f("work_address_format4"),
				f("work_address_format5"),
				f("work_address_format6"),
				f("work_address_format7"),
				f("work_address_format8"),
				f("work_address_format9")];
	} else {
	    // Fall-back address formats if no localization is defined
	    sab_home_address_fmt = [
		  "{NameFL}",                          // Name
		  "{HomeAddress}",                          // Address
		  "{HomeAddress2}",              
		  "{HomeCity}, {HomeState} {HomeZipCode}",  // City, State Zip
		  "{HomeCountry}"];                         // Country
	    
	    sab_work_address_fmt = [
		  "{NameFL}",                          // First Last Name
		  "{JobTitle}",                             // Title
		  "{Department}",                           // Department
		  "{Company}",                              // Company
		  "{WorkAddress}",                          // Address
		  "{WorkAddress2}",              
		  "{WorkCity}, {WorkState} {WorkZipCode}",  // City, State Zip
		  "{WorkCountry}"];                         // Country
	}
    }


    if (type == "home") { return sab_home_address_fmt; }
    if (type == "work") { return sab_work_address_fmt; }
}



//
// selectAddresBookCopy - copy a given field to the clipboard
//
// field - string designating which field to copy from address card.
//         Possible values:
//           * any property value in the card itself, see
//             https://developer.mozilla.org/en/nsIAbCard%2f%2fThunderbird3
//           * "DisplayNameFL" - display name as First,Last
//           * "DisplayNameLF" - display name as Last,First
//           * "FullHomeAddress" or "FullWorkAddress" - address mailing label
//           * "NameHomeAddress" or "NameWorkAddress" - name&address mailing label
// Returns: nothing
//
function selectAddressBookCopy(field)
{
  var cards = GetSelectedAbCards();
  var count = cards.length;
  var str = "";
  if (count != 1) { return; }
  var realCard = cards[0];
  var nameFL, nameLF;
  var accessor;

  // Deal with differences between TB2 and TB3
  try {
      nameFL = realCard.generateName(2);
      nameLF = realCard.generateName(1);
      accessor = (function (prop) { return realCard.getProperty(prop, ""); } );
  } catch (e) {
      nameFL = gAddrbookSession.generateNameFromCard(realCard, 2);
      nameLF = gAddrbookSession.generateNameFromCard(realCard, 1);
      accessor = (function (prop) { return realCard.getCardValue(prop) || ""; } );
  }
  
  var card = { getProperty : (function (prop) {
	         return this[prop] || accessor(prop); }),
	       DisplayNameFL: nameFL,
	       DisplayNameLF: nameLF,
               primaryEmail : realCard.primaryEmail,
               displayName : realCard.displayName,
               isMailList : realCard.isMailList,
               mailListURI : realCard.mailListURI
  };

  if (field == "NameHomeAddress" || field == "NameWorkAddress") {
      card.NameFL = nameFL;
      card.NameLF = nameLF;
  }


  if (field == "FullHomeAddress" || field == "NameHomeAddress" ||
      field == "FullWorkAddress" || field == "NameWorkAddress") {
      if (field == "FullHomeAddress" || field == "NameHomeAddress") {
	  fmt = sab_address_fmt("home");
      } else {
	  fmt = sab_address_fmt("work");
      }

      // Replace values from the format according to:
      //                   {PARNAME.}SEPSTR..
      var re = new RegExp('{([^{}]*)}([^{}]*)','g');
      var fs = (function(m,parname,sepstr) {
	        var r = card.getProperty(parname);
		if (r && r != "") { 
		    return r+sepstr;
		} else { return ""; }});

      str = fmt.join("\n\n");
      // Perform the magical tranformation
      str = str.replace(re,fs);

      // Remove multiple, leading and trailing carriage returns...
      str = str.replace(/\n+/g, "\n")  
      str = str.replace(/^\n*/g,"");
      str = str.replace(/\n*$/g,"");
      // ... and add one final trailing carriage return
      str = str + '\n';
  } else {
      // OK, nothing special, just copy the field straight from
      // the Address book.
      str = card.getProperty(field);
  }

  if (str == "") return;

  const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
  if (!gClipboardHelper) return;
  gClipboardHelper.copyString(str);

  return;
}

//////////////////////////////////////////////////////////////
// 
// selectAddressBookTrimmer - this code will slightly trim
//   the address card HTML so that there are not extraneous spaces
//   or newline characters.  This makes it easier to select, copy
//   and paste well-formatted entries without a lot of crufty
//   formatting stuff.
//
// The function is installed "onLoad", and goes through all the 
// description nodes with class CardViewText or CardViewLink.
// If there are any child text nodes with white space, the white
// space is removed, and a trailing text node with a newline "\n"
// character is added.
//
function selectAddressBookTrimmer()
{
    var i;
    var els = document.getElementsByTagName('description');

    for (i=0; i<els.length; i++) {
	var el = els[i];
	if (el.className == "CardViewText" || el.className == "CardViewLink") {
	    // Clear out all the incidental text nodes that have 
	    // annoying white space...
	    var j;
	    for (j=0; j<el.childNodes.length; j++) {
		var c = el.childNodes[j];
		if (c.nodeName == "#text" && c.textContent.match(/^\s*$/)) {
		    c.textContent = "";
		}
	    }

	    // ... append a line-feed so that copy-paste operations
	    // have something pretty to work with.
	    // Work around the retarded lack of insertAfter().
	    // Also, since abCardViewOverlay retardedly demands that
	    // there be only a single daughter text node, we cannot
	    // use appendChild().
	    t = document.createTextNode("\n");
	    if (el.nextSibling) {
		el.parentNode.insertBefore(t, el.nextSibling);
	    } else {
		el.parentNode.appendChild(t);
	    }
	}
    }
}

window.addEventListener('load', selectAddressBookTrimmer, false);
